'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _settings = require('./constants/settings');

var _settings2 = _interopRequireDefault(_settings);

var _blacklist = require('./blacklist');

var _blacklist2 = _interopRequireDefault(_blacklist);

var _links = require('./links');

var _links2 = _interopRequireDefault(_links);

var _pool = require('./pool');

var _pool2 = _interopRequireDefault(_pool);

var _uniq = require('./misc/uniq');

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

var _puppeteer = require('puppeteer');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Crawlefier = function () {
    function Crawlefier() {
        var settings = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _settings2.default;
        var blackList = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

        _classCallCheck(this, Crawlefier);

        this.settings = {
            timeout: settings.timeout,
            userAgent: settings.userAgent,
            headlessChrome: settings.headlessChrome,
            depth: {
                current: 0,
                max: settings.depth
            }
        };

        this.counter = {
            links: {
                all: 0,
                done: 0
            }
        };

        this.blackList = new _blacklist2.default(blackList);
        this.linksList = new _links2.default();
        this.pool = new _pool2.default(settings.threads);
        this.getLinksMethod = settings.external ? this.allLinks : this.internalLinks;
        this.getRequestMethod = settings.headlessChrome ? this.puppeteerRequest : this.axiosRequest;
    }

    _createClass(Crawlefier, [{
        key: 'setCallbacks',
        value: function setCallbacks(onSuccess, onError) {
            if (onSuccess) this.onSuccessCallback = onSuccess;
            if (onError) this.onErrorCallback = onError;
        }
    }, {
        key: 'launch',
        value: function () {
            var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(links, onSuccess, onError) {
                var _this = this;

                var urls;
                return regeneratorRuntime.wrap(function _callee$(_context) {
                    while (1) {
                        switch (_context.prev = _context.next) {
                            case 0:
                                urls = this.normalizeLinks(links);

                                if (!(urls == null)) {
                                    _context.next = 3;
                                    break;
                                }

                                throw new Error('No links!');

                            case 3:

                                this.setCallbacks(onSuccess, onError);

                                if (!this.settings.headlessChrome) {
                                    _context.next = 8;
                                    break;
                                }

                                _context.next = 7;
                                return (0, _puppeteer.launch)();

                            case 7:
                                this.headlessChrome = _context.sent;

                            case 8:
                                return _context.abrupt('return', new Promise(function (resolve, reject) {
                                    _this.reject = reject;
                                    _this.resolve = resolve;

                                    _this.linksList.set(urls);
                                    _this.crawl(urls);
                                }));

                            case 9:
                            case 'end':
                                return _context.stop();
                        }
                    }
                }, _callee, this);
            }));

            function launch(_x3, _x4, _x5) {
                return _ref.apply(this, arguments);
            }

            return launch;
        }()

        /**
         * Normalize urls without protocol.
         * @param {array} links 
         * @returns {array} urls with protocol.
         */

    }, {
        key: 'normalizeLinks',
        value: function normalizeLinks(links) {
            return links.map(function (link) {
                return link.match(/http:\/\/|https:\/\//g) ? link : 'http://' + link;
            });
        }
    }, {
        key: 'crawl',
        value: function crawl(links) {
            var _this2 = this;

            if (links.length > 0) {
                this.settings.depth.current++;
                this.parsedLinks = [];
                this.counter.links.done = 0;
                this.counter.links.all = links.length;

                links.forEach(function (link) {
                    return _this2.pool.addTask(_this2.getRequestMethod.bind(_this2, link));
                });

                this.pool.run();
            } else {
                this.resolve(this.linksList.get());
            }
        }
    }, {
        key: 'isDone',
        value: function isDone() {
            return this.counter.links.done == this.counter.links.all;
        }
    }, {
        key: 'wait',
        value: function wait(ms) {
            return new Promise(function (resolve) {
                return setTimeout(resolve, ms);
            });
        }
    }, {
        key: 'isCloudflare',
        value: function isCloudflare(content) {
            if (content.match(new RegExp('cloudflare', 'gi'))) {
                return true;
            }
        }
    }, {
        key: 'onSuccess',
        value: function onSuccess(url, content) {
            this.parsedLinks = this.parsedLinks.concat(this.getLinksMethod(url, content));
            this.update();

            if (this.onSuccessCallback) {
                this.onSuccessCallback({ url: url, content: content });
            }
        }
    }, {
        key: 'onError',
        value: function onError(url, error) {
            this.update();

            if (this.onErrorCallback) {
                this.onErrorCallback({ url: url, message: error });
            }
        }
    }, {
        key: 'puppeteerRequest',
        value: function () {
            var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(url) {
                var page, content;
                return regeneratorRuntime.wrap(function _callee2$(_context2) {
                    while (1) {
                        switch (_context2.prev = _context2.next) {
                            case 0:
                                _context2.prev = 0;
                                _context2.next = 3;
                                return this.headlessChrome.newPage();

                            case 3:
                                page = _context2.sent;
                                _context2.next = 6;
                                return page.setUserAgent(this.settings.userAgent);

                            case 6:
                                _context2.next = 8;
                                return page.goto(url, { timeout: this.settings.timeout });

                            case 8:
                                _context2.next = 10;
                                return page.content();

                            case 10:
                                content = _context2.sent;

                                if (!this.isCloudflare(content)) {
                                    _context2.next = 17;
                                    break;
                                }

                                _context2.next = 14;
                                return this.wait(7500);

                            case 14:
                                _context2.next = 16;
                                return page.content();

                            case 16:
                                content = _context2.sent;

                            case 17:

                                this.onSuccess(url, content);
                                _context2.next = 20;
                                return page.close();

                            case 20:
                                _context2.next = 25;
                                break;

                            case 22:
                                _context2.prev = 22;
                                _context2.t0 = _context2['catch'](0);

                                this.onError(url, _context2.t0);

                            case 25:
                            case 'end':
                                return _context2.stop();
                        }
                    }
                }, _callee2, this, [[0, 22]]);
            }));

            function puppeteerRequest(_x6) {
                return _ref2.apply(this, arguments);
            }

            return puppeteerRequest;
        }()
    }, {
        key: 'axiosRequest',
        value: function () {
            var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(url) {
                var req;
                return regeneratorRuntime.wrap(function _callee3$(_context3) {
                    while (1) {
                        switch (_context3.prev = _context3.next) {
                            case 0:
                                _context3.prev = 0;
                                _context3.next = 3;
                                return _axios2.default.get(url, { timeout: this.settings.timeout, responseType: 'text', userAgent: this.settings.userAgent });

                            case 3:
                                req = _context3.sent;

                                this.onSuccess(url, req.data);
                                _context3.next = 10;
                                break;

                            case 7:
                                _context3.prev = 7;
                                _context3.t0 = _context3['catch'](0);

                                this.onError(url, _context3.t0);

                            case 10:
                            case 'end':
                                return _context3.stop();
                        }
                    }
                }, _callee3, this, [[0, 7]]);
            }));

            function axiosRequest(_x7) {
                return _ref3.apply(this, arguments);
            }

            return axiosRequest;
        }()
    }, {
        key: 'update',
        value: function update() {
            this.counter.links.done++;
            this.parsedLinks = (0, _uniq.uniq)(this.blackList.filter(this.linksList.filterNextLinks(this.parsedLinks)));
            this.linksList.set(this.parsedLinks);

            if (this.isDone()) {
                if (this.settings.depth.current < this.settings.depth.max) {
                    this.crawl(this.parsedLinks);
                } else {
                    if (this.settings.headlessChrome) {
                        this.headlessChrome.close();
                    }

                    this.resolve(this.linksList.get());
                }
            } else {
                this.pool.nextTask();
            }
        }

        /**
         * 
         * @param {string} url
         * @returns {array} url protocol, host, path
         * 
         * Exeption for broken links.
         * 
         */

    }, {
        key: 'getUrlProps',
        value: function getUrlProps(url) {
            try {
                var match = url.match(/^(http|https)?(?:[\:\/]*)([a-z0-9\.-]*)(?:\:([0-9]+))?(\/[^?#]*)?(?:\?([^#]*))?(?:#(.*))?$/i);

                return {
                    protocol: match[1],
                    host: match[2],
                    path: match[4]
                };
            } catch (error) {
                return {
                    protocol: null,
                    host: null,
                    path: null
                };
            }
        }
    }, {
        key: 'parseLinks',
        value: function parseLinks(content) {
            try {
                return content.match(new RegExp('<a(.*?)href=(((\'|").*?(\'|")))', 'gi')).join("\n").match(new RegExp('href=(((\'|").*?(\'|")))', 'gi')).map(function (item) {
                    return item.slice(6, -1);
                });
            } catch (error) {
                return [];
            }
        }
    }, {
        key: 'allLinks',
        value: function allLinks(url, content) {
            var _this3 = this;

            var urlProps = this.getUrlProps(url),
                extractLinks = this.parseLinks(content);

            return extractLinks.map(function (link) {
                if (link[0] == "/" && link[1] == "/") {
                    return link.slice(2);
                } else {
                    if (link[0] == "/") {
                        return urlProps.protocol + "://" + urlProps.host + link;
                    }

                    if (!_this3.getUrlProps(link).protocol) {
                        return urlProps.protocol + "://" + urlProps.host + "/" + link;
                    }
                }

                return link;
            });
        }
    }, {
        key: 'internalLinks',
        value: function internalLinks(url, content) {
            var _this4 = this;

            var urlProps = this.getUrlProps(url),
                extractLinks = this.parseLinks(content);

            var filtered = extractLinks.filter(function (link) {
                if (link[0] == "/") {
                    return link;
                }

                if (urlProps.host === _this4.getUrlProps(link).host) {
                    return link;
                }
            });

            var next = filtered.map(function (link) {
                if (link[0] == "/") {
                    return urlProps.protocol + "://" + urlProps.host + link;
                }

                if (urlProps.host === _this4.getUrlProps(link).host) {
                    if (link[0] == "/" && link[1] == "/") {
                        return link.slice(2);
                    } else {
                        if (link[0] == "/") {
                            return urlProps.protocol + "://" + urlProps.host + link;
                        }

                        if (!_this4.getUrlProps(link).protocol) {
                            return urlProps.protocol + "://" + urlProps.host + "/" + link;
                        }
                    }

                    return link;
                }
            });

            return next;
        }
    }]);

    return Crawlefier;
}();

exports.default = Crawlefier;